import java.io.*;

//
// a class representing an RGB color tuple and some simple operations on it
//
class ColorRGB implements Serializable {
    public ColorRGB(double r, double g, double b) {
        set(r,g,b);
    }

    public ColorRGB(int rgb) {
        setRGB(rgb);
    }

    public ColorRGB(ColorRGB c) {
        set(c);
    }

    public double getComponent(int i) {
        return rgb[i];
    }

    public void setComponent(int i, double value) {
        rgb[i] = value;
    }

    public void update(ColorRGB c, double alpha) {
        rgb[0] = rgb[0]*(1-alpha) + c.rgb[0]*alpha;
        rgb[1] = rgb[1]*(1-alpha) + c.rgb[1]*alpha;
        rgb[2] = rgb[2]*(1-alpha) + c.rgb[2]*alpha;
    }

    public double distance(double r, double g, double b) {
        double r1 = rgb[0]; 
        double g1 = rgb[1];
        double b1 = rgb[2];
        return Math.sqrt((r1-r)*(r1-r)+(g1-g)*(g1-g)+(b1-b)*(b1-b));
    }

    public double distance(ColorRGB c) {
        return distance(c.rgb[0], c.rgb[1], c.rgb[2]);
    }

    public double distance(int rgb) {
        return distance( (0x00FF0000 & rgb) >>> 16, (0x0000FF00 & rgb) >>> 8, (0x000000FF & rgb));
    }
    
    public ColorRGB plus(ColorRGB c) {
        return new ColorRGB(rgb[0] + c.rgb[0], rgb[1] + c.rgb[1], rgb[2] + c.rgb[2]);
    }

    public ColorRGB times(double s) {
        return new ColorRGB(s*rgb[0], s*rgb[1], s*rgb[2]);
    }

    public int getRGB() {
        return (0xFF << 24) | 
               ((int)Math.round(Math.min(Math.max(rgb[0],0),255)) << 16) | 
               ((int)Math.round(Math.min(Math.max(rgb[1],0),255)) << 8) | 
                (int)Math.round(Math.min(Math.max(rgb[2],0),255));
    }
    public void setRGB(int rgb) { 
        this.rgb[0] = (0x00FF0000 & rgb) >>> 16;
        this.rgb[1] = (0x0000FF00 & rgb) >>> 8;
        this.rgb[2] = (0x000000FF & rgb);
    }

    public void set(double r, double g, double b) {
        rgb[0] = r;
        rgb[1] = g;
        rgb[2] = b;
    }

    public void set(ColorRGB c) {
        rgb[0] = c.rgb[0];
        rgb[1] = c.rgb[1];
        rgb[2] = c.rgb[2];
    }

    public String toString() {
        return "[" + rgb[0] + "," + rgb[1] + "," + rgb[2] + "]";
    }

    private double rgb[] = {0, 0, 0};
}
