import java.io.*;
import java.awt.image.*;

//
// a class representing a small texture patch for RGB pixels and some simple operations on it
//
class Patch {
    public Patch(BufferedImage parentImage, int x, int y, int w, int h) {
        if (x < 0) {
            w += x;
            x = 0;
        } else if (x+w >= parentImage.getWidth()) {
            w = parentImage.getWidth()-1-x;
        }
            
        if (y < 0) {
            h += y;
            y = 0;
        } else if (y+h >= parentImage.getHeight()) {
            h = parentImage.getHeight()-1-y;
        }

        this.parentImage = parentImage;
        this.x = x;
        this.y = y;
        this.w = w;
        this.h = h;

        data = new int[w*h];
        parentImage.getRGB(x, y, w, h, data, 0, w);
    }

    BufferedImage parentImage;
    int x, y;

    int data[];
    int w, h;

    int get(int x, int y) {
        return data[y*w+x];
    }

    void set(int x, int y, int d) {
        data[y*w+x] = d;
    }

    public double distance(Patch p) {
        ColorRGB c = new ColorRGB(0);
        double d = 0;
        for (int j = 0; j < w && j < p.w; j++) {
            for (int i = 0; i < h && i < p.h; i++) {
                c.setRGB(data[i*w+j]);
                d += c.distance(p.data[i*p.w+j]);
            }
        } 
        
        return d;
    }

    public boolean equals(Object obj) {
        Patch p = (Patch)obj;
        if (parentImage == p.parentImage && x == p.x && y == p.y)
            return true;
        else
            return false;
    }
}
